package cn.com.taiji.beidou.web.controller.system;

import cn.com.taiji.beidou.common.constant.Constants;
import cn.com.taiji.beidou.common.core.domain.AjaxResult;
import cn.com.taiji.beidou.common.core.domain.R;
import cn.com.taiji.beidou.common.core.domain.entity.SysMenu;
import cn.com.taiji.beidou.common.core.domain.entity.SysUser;
import cn.com.taiji.beidou.common.core.domain.model.LoginBody;
import cn.com.taiji.beidou.common.core.domain.model.LoginUser;
import cn.com.taiji.beidou.common.utils.SecurityUtils;
import cn.com.taiji.beidou.framework.web.service.SysLoginService;
import cn.com.taiji.beidou.framework.web.service.SysPermissionService;
import cn.com.taiji.beidou.system.service.ISysMenuService;
import cn.com.taiji.beidou.system.service.ISysUserService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;

/**
 * 登录验证
 * 
 * @author ruoyi
 */
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;


    /**
     * 登录方法
     * 
     * @param loginBody 登录信息
     * @return 结果
     */
    @ApiOperation("用户名密码登录")
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody)
    {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    @ApiOperation("手机号+短信验证码登录")
    @GetMapping("/loginByCode")
    public R<LoginUser> loginByCode(String mobile, String code){

        return loginService.loginByCode(mobile,code);
    }


    /**
     * 获取用户信息
     * 
     * @return 用户信息
     */
    @ApiOperation("获取用户信息")
    @GetMapping("getInfo")
    public AjaxResult getInfo()
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }

    /**
     * 获取路由信息
     * 
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }

    @ApiOperation("获取短信验证码")
    @GetMapping("/getCode")
    public R getCode(String telephone)
    {
        return loginService.getCode(telephone);
    }
}
